=== modified file 'include/core/action.h'
--- include/core/action.h	2012-01-24 14:57:05 +0000
+++ include/core/action.h	2012-01-31 14:52:20 +0000
@@ -170,7 +170,7 @@
 
 	bool active ();
 
-	friend class CompScreen;
+	friend class CompScreenImpl;
 
     private:
 	PrivateAction *priv;

=== modified file 'include/core/modifierhandler.h'
--- include/core/modifierhandler.h	2010-03-24 09:37:19 +0000
+++ include/core/modifierhandler.h	2012-01-31 14:52:20 +0000
@@ -98,7 +98,7 @@
 	 */
 	const XModifierKeymap * modMap ();
 
-	friend class CompScreen;
+	friend class CompScreenImpl;
 
     private:
 

=== modified file 'include/core/option.h'
--- include/core/option.h	2012-01-26 09:11:19 +0000
+++ include/core/option.h	2012-01-31 14:52:20 +0000
@@ -40,7 +40,7 @@
 
 class CompAction;
 class CompMatch;
-class CompScreen;
+class CompScreenImpl;
 
 
 /**

=== modified file 'include/core/plugin.h'
--- include/core/plugin.h	2012-01-30 16:42:19 +0000
+++ include/core/plugin.h	2012-01-31 14:52:20 +0000
@@ -32,13 +32,15 @@
 
 #include <string.h>
 
-class AbstractCompScreen;
+//#define ARG_ABSTRACT_COMP_SCREEN
+
 #if defined(ARG_ABSTRACT_COMP_SCREEN)
-class AbstractCompScreen;
-extern AbstractCompScreen   *screen;
+class CompScreen;
+extern CompScreen   *screen;
 #else
 class CompScreen;
-extern CompScreen   *screen;
+class CompScreenImpl;
+extern CompScreenImpl   *screen;
 #endif
 
 #include <map>
@@ -170,9 +172,9 @@
 
     public:
 
-	static bool screenInitPlugins (CompScreen *s);
+	static bool screenInitPlugins (CompScreenImpl *s);
 
-	static void screenFiniPlugins (CompScreen *s);
+	static void screenFiniPlugins (CompScreenImpl *s);
 
 	static bool windowInitPlugins (CompWindow *w);
 

=== modified file 'include/core/screen.h'
--- include/core/screen.h	2012-01-30 16:42:19 +0000
+++ include/core/screen.h	2012-01-31 14:52:20 +0000
@@ -38,9 +38,10 @@
 #include <core/modifierhandler.h>
 #include <core/valueholder.h>
 
-class CompScreen;
+class CompScreenImpl;
 class PrivateScreen;
 class CompManager;
+class CoreWindow;
 
 typedef std::list<CompWindow *> CompWindowList;
 typedef std::vector<CompWindow *> CompWindowVector;
@@ -51,10 +52,12 @@
 extern bool       noDetection;
 extern bool       debugOutput;
 
+//#define ARG_ABSTRACT_COMP_SCREEN
+
 #if defined(ARG_ABSTRACT_COMP_SCREEN)
-extern AbstractCompScreen   *screen;
+extern CompScreen   *screen;
 #else
-extern CompScreen   *screen;
+extern CompScreenImpl   *screen;
 #endif
 
 extern ModifierHandler *modHandler;
@@ -145,9 +148,11 @@
 
 };
 
-class AbstractCompScreen :
+class CompScreen :
     public WrapableHandler<ScreenInterface, 18>,
-    public CompSize
+    public PluginClassStorage,
+    public CompSize,
+    public CompOption::Class
 {
 public:
     WRAPABLE_HND (0, ScreenInterface, void, fileWatchAdded, CompFileWatch *)
@@ -188,6 +193,9 @@
     WRAPABLE_HND (17, ScreenInterface, void, addSupportedAtoms,
 		  std::vector<Atom>& atoms);
 
+    static unsigned int allocPluginClassIndex ();
+    static void freePluginClassIndex (unsigned int index);
+
     // Interface hoisted from CompScreen
     virtual bool updateDefaultIcon () = 0;
     virtual Display * dpy () = 0;
@@ -214,6 +222,68 @@
 
     virtual bool grabsEmpty() const = 0;
     virtual void sizePluginClasses(unsigned int size) = 0;
+    virtual CompOutput::vector & outputDevs () = 0;
+    virtual void setWindowState (unsigned int state, Window id) = 0;
+    virtual bool XShape () = 0;
+    virtual std::vector<XineramaScreenInfo> & screenInfo () = 0;
+    virtual CompWindowList & serverWindows () = 0;
+    virtual void setWindowProp (Window       id,
+			    Atom         property,
+			    unsigned int value) = 0;
+    virtual Window activeWindow () = 0;
+    virtual unsigned int currentDesktop () = 0;
+    virtual CompActiveWindowHistory *currentHistory () = 0;
+    virtual void focusDefaultWindow () = 0;
+    virtual Time getCurrentTime () = 0;
+    virtual unsigned int getWindowProp (Window       id,
+				    Atom         property,
+				    unsigned int defaultValue) = 0;
+    virtual void insertServerWindow (CompWindow *w, Window aboveId) = 0;
+    virtual void insertWindow (CompWindow *w, Window aboveId) = 0;
+    virtual unsigned int nDesktop () = 0;
+    virtual int outputDeviceForGeometry (const CompWindow::Geometry& gm) = 0;
+    virtual int screenNum () = 0;
+    virtual void unhookServerWindow (CompWindow *w) = 0;
+    virtual void unhookWindow (CompWindow *w) = 0;
+    virtual void viewportForGeometry (const CompWindow::Geometry &gm,
+				  CompPoint                   &viewport) = 0;
+
+    virtual void removeFromCreatedWindows(CoreWindow *cw) = 0;
+    virtual void addToDestroyedWindows(CompWindow * cw) = 0;
+    virtual const CompRect & workArea () const = 0;
+    virtual void removeAction (CompAction *action) = 0;
+    virtual CompOption::Vector & getOptions () = 0;
+    virtual bool setOption (const CompString &name, CompOption::Value &value) = 0;
+    virtual void storeValue (CompString key, CompPrivate value) = 0;
+    virtual bool hasValue (CompString key) = 0;
+    virtual CompPrivate getValue (CompString key) = 0;
+    virtual void eraseValue (CompString key) = 0;
+    virtual CompWatchFdHandle addWatchFd (int             fd,
+				      short int       events,
+				      FdWatchCallBack callBack) = 0;
+    virtual void removeWatchFd (CompWatchFdHandle handle) = 0;
+    virtual void eventLoop () = 0;
+
+    virtual CompFileWatchHandle addFileWatch (const char        *path,
+					  int               mask,
+					  FileWatchCallBack callBack) = 0;
+    virtual void removeFileWatch (CompFileWatchHandle handle) = 0;
+    virtual const CompFileWatchList& getFileWatches () const = 0;
+    virtual void updateSupportedWmHints () = 0;
+
+	friend class CompTimer;
+	friend class CompWindow;
+	friend class PrivateWindow;
+	friend class CoreWindow;
+	friend class ModifierHandler;
+	friend class CompEventSource;
+	friend class CompTimeoutSource;
+	friend class CompManager;
+	friend class CompWatchFd;
+
+protected:
+	CompScreen();
+	PrivateScreen *priv;
 
 private:
     // The "wrapable" functions delegate to these (for mocking)
@@ -242,18 +312,15 @@
  * A wrapping of the X display screen. This takes care of communication to the
  * X server.
  */
-class CompScreen :
-    public AbstractCompScreen,
-    public PluginClassStorage,
-    public CompOption::Class
+class CompScreenImpl : public CompScreen
 {
 
     public:
 	typedef void* GrabHandle;
 
     public:
-	CompScreen ();
-	~CompScreen ();
+	CompScreenImpl ();
+	~CompScreenImpl ();
 
 	bool init (const char *name);
 
@@ -462,27 +529,13 @@
 
 	void updateSupportedWmHints ();
 
-	static unsigned int allocPluginClassIndex ();
-	static void freePluginClassIndex (unsigned int index);
-
 	unsigned int showingDesktopMask() const;
-
 	virtual bool grabsEmpty() const;
-
 	virtual void sizePluginClasses(unsigned int size);
-
-	friend class CompTimer;
-	friend class CompWindow;
-	friend class PrivateWindow;
-	friend class CoreWindow;
-	friend class ModifierHandler;
-	friend class CompEventSource;
-	friend class CompTimeoutSource;
-	friend class CompManager;
-	friend class CompWatchFd;
-
-    private:
-	PrivateScreen *priv;
+	virtual void setWindowState (unsigned int state, Window id);
+	virtual void removeFromCreatedWindows(CoreWindow *cw);
+	virtual void addToDestroyedWindows(CompWindow * cw);
+
 
     public :
 

=== modified file 'include/core/window.h'
--- include/core/window.h	2012-01-20 15:20:44 +0000
+++ include/core/window.h	2012-01-31 14:52:20 +0000
@@ -539,7 +539,7 @@
 	WRAPABLE_HND (19, WindowInterface, bool, focused);
 
 	friend class PrivateWindow;
-	friend class CompScreen;
+	friend class CompScreenImpl;
 	friend class PrivateScreen;
 	friend class ModifierHandler;
 	friend class CoreWindow;

=== modified file 'plugins/annotate/src/annotate.h'
--- plugins/annotate/src/annotate.h	2012-01-18 16:26:45 +0000
+++ plugins/annotate/src/annotate.h	2012-01-31 14:52:20 +0000
@@ -70,7 +70,7 @@
 	CompositeScreen *cScreen;
 	GLScreen *gScreen;
 
-	CompScreen::GrabHandle grabIndex;
+	CompScreenImpl::GrabHandle grabIndex;
 
 	Pixmap pixmap;
 	GLTexture::List texture;

=== modified file 'plugins/clone/src/clone.h'
--- plugins/clone/src/clone.h	2012-01-18 16:26:45 +0000
+++ plugins/clone/src/clone.h	2012-01-31 14:52:20 +0000
@@ -61,7 +61,7 @@
 	CompositeScreen *cScreen;
 	GLScreen	*gScreen;
 
-	CompScreen::GrabHandle grabHandle;
+	CompScreenImpl::GrabHandle grabHandle;
 	bool		       grab;
 
 	float offset;

=== modified file 'plugins/compiztoolbox/include/compiztoolbox/compiztoolbox.h'
--- plugins/compiztoolbox/include/compiztoolbox/compiztoolbox.h	2012-01-18 16:26:45 +0000
+++ plugins/compiztoolbox/include/compiztoolbox/compiztoolbox.h	2012-01-31 14:52:20 +0000
@@ -99,7 +99,7 @@
 	CompWindow   *selectedWindow;
 	unsigned int lastActiveNum;
 
-	CompScreen::GrabHandle grabIndex;
+	CompScreenImpl::GrabHandle grabIndex;
 
 	bool moreAdjust;
 

=== modified file 'plugins/composite/src/screen.cpp'
--- plugins/composite/src/screen.cpp	2012-01-23 18:41:06 +0000
+++ plugins/composite/src/screen.cpp	2012-01-31 14:52:20 +0000
@@ -372,14 +372,14 @@
     if (priv->pHnd)
 	return false;
 
-    CompScreen::checkForError (dpy);
+    CompScreenImpl::checkForError (dpy);
 
     XCompositeRedirectSubwindows (dpy, screen->root (),
 				  CompositeRedirectManual);
 
     priv->overlayWindowCount = 0;
 
-    if (CompScreen::checkForError (dpy))
+    if (CompScreenImpl::checkForError (dpy))
     {
 	compLogMessage ("composite", CompLogLevelError,
 			"Another composite manager is already "

=== modified file 'plugins/composite/src/window.cpp'
--- plugins/composite/src/window.cpp	2012-01-16 09:50:28 +0000
+++ plugins/composite/src/window.cpp	2012-01-31 14:52:20 +0000
@@ -31,7 +31,7 @@
     PluginClassHandler<CompositeWindow, CompWindow, COMPIZ_COMPOSITE_ABI> (w),
     priv (new PrivateCompositeWindow (w, this))
 {
-    CompScreen *s = screen;
+    CompScreenImpl *s = screen;
 
     if (w->windowClass () != InputOnly)
     {

=== modified file 'plugins/cube/src/privates.h'
--- plugins/cube/src/privates.h	2010-10-01 13:42:31 +0000
+++ plugins/cube/src/privates.h	2012-01-31 14:52:20 +0000
@@ -119,7 +119,7 @@
 	GLfloat  mDistance;
 	GLfloat  mTc[12];
 
-	CompScreen::GrabHandle mGrabIndex;
+	CompScreenImpl::GrabHandle mGrabIndex;
 
 	int mSrcOutput;
 

=== modified file 'plugins/dbus/src/dbus.cpp'
--- plugins/dbus/src/dbus.cpp	2011-07-06 19:24:38 +0000
+++ plugins/dbus/src/dbus.cpp	2012-01-31 14:52:20 +0000
@@ -271,7 +271,7 @@
 				   	       DBusMessage    *message,
 				   	       char           **path)
 {
-    CompScreen *s;
+    CompScreenImpl *s;
     char screenName[256];
 
     xmlTextWriterPtr writer;

=== modified file 'plugins/decor/src/decor.cpp'
--- plugins/decor/src/decor.cpp	2012-01-21 19:05:23 +0000
+++ plugins/decor/src/decor.cpp	2012-01-31 14:52:20 +0000
@@ -1963,11 +1963,11 @@
 	memcpy (&dmWin, data, sizeof (Window));
 	XFree (data);
 
-	CompScreen::checkForError (screen->dpy ());
+	CompScreenImpl::checkForError (screen->dpy ());
 
 	XGetWindowAttributes (screen->dpy (), dmWin, &attr);
 
-	if (CompScreen::checkForError (screen->dpy ()))
+	if (CompScreenImpl::checkForError (screen->dpy ()))
 	    dmWin = None;
 	else
 	{

=== modified file 'plugins/move/src/move.cpp'
--- plugins/move/src/move.cpp	2012-01-20 14:42:53 +0000
+++ plugins/move/src/move.cpp	2012-01-31 14:52:20 +0000
@@ -54,7 +54,7 @@
 	int          x, y, button;
 	bool         sourceExternalApp;
 
-	CompScreen *s = screen;
+	CompScreenImpl *s = screen;
 
 	mods = CompOption::getIntOptionNamed (options, "modifiers", 0);
 
@@ -210,7 +210,7 @@
 /* creates a region containing top and bottom struts. only struts that are
    outside the screen workarea are considered. */
 static Region
-moveGetYConstrainRegion (CompScreen *s)
+moveGetYConstrainRegion (CompScreenImpl *s)
 {
     CompWindow   *w;
     Region       region;
@@ -299,7 +299,7 @@
 }
 
 static void
-moveHandleMotionEvent (CompScreen *s,
+moveHandleMotionEvent (CompScreenImpl *s,
 		       int	  xRoot,
 		       int	  yRoot)
 {

=== modified file 'plugins/move/src/move.h'
--- plugins/move/src/move.h	2012-01-18 17:14:59 +0000
+++ plugins/move/src/move.h	2012-01-31 14:52:20 +0000
@@ -82,7 +82,7 @@
 
 	GLushort moveOpacity;
 
-	CompScreen::GrabHandle grab;
+	CompScreenImpl::GrabHandle grab;
 
 	Cursor moveCursor;
 

=== modified file 'plugins/resize/src/resize.h'
--- plugins/resize/src/resize.h	2012-01-18 16:26:45 +0000
+++ plugins/resize/src/resize.h	2012-01-31 14:52:20 +0000
@@ -128,7 +128,7 @@
 	int          pointerDy;
 	KeyCode      key[NUM_KEYS];
 
-	CompScreen::GrabHandle grabIndex;
+	CompScreenImpl::GrabHandle grabIndex;
 
 	Cursor leftCursor;
 	Cursor rightCursor;

=== modified file 'plugins/rotate/src/rotate.h'
--- plugins/rotate/src/rotate.h	2012-01-18 16:26:45 +0000
+++ plugins/rotate/src/rotate.h	2012-01-31 14:52:20 +0000
@@ -116,7 +116,7 @@
 	bool                   mSnapTop;
 	bool                   mSnapBottom;
 
-	CompScreen::GrabHandle mGrabIndex;
+	CompScreenImpl::GrabHandle mGrabIndex;
 
 	GLfloat                mXrot, mXVelocity;
 	GLfloat                mYrot, mYVelocity;

=== modified file 'plugins/scale/src/privates.h'
--- plugins/scale/src/privates.h	2011-10-14 13:18:15 +0000
+++ plugins/scale/src/privates.h	2012-01-31 14:52:20 +0000
@@ -115,7 +115,7 @@
 	KeyCode	 leftKeyCode, rightKeyCode, upKeyCode, downKeyCode;
 
 	bool grab;
-	CompScreen::GrabHandle grabIndex;
+	CompScreenImpl::GrabHandle grabIndex;
 
 	Window dndTarget;
 

=== modified file 'plugins/screenshot/src/screenshot.h'
--- plugins/screenshot/src/screenshot.h	2012-01-18 16:26:45 +0000
+++ plugins/screenshot/src/screenshot.h	2012-01-31 14:52:20 +0000
@@ -63,7 +63,7 @@
 	CompositeScreen *cScreen;
 	GLScreen        *gScreen;
 
-	CompScreen::GrabHandle mGrabIndex;
+	CompScreenImpl::GrabHandle mGrabIndex;
 	Bool                   mGrab;
 
 	int  mX1, mY1, mX2, mY2;

=== modified file 'plugins/water/src/water.h'
--- plugins/water/src/water.h	2012-01-18 16:26:45 +0000
+++ plugins/water/src/water.h	2012-01-31 14:52:20 +0000
@@ -102,7 +102,7 @@
 
 	float offsetScale;
 
-	CompScreen::GrabHandle grabIndex;
+	CompScreenImpl::GrabHandle grabIndex;
 
 	int width, height;
 

=== modified file 'src/actions.cpp'
--- src/actions.cpp	2012-01-30 16:42:19 +0000
+++ src/actions.cpp	2012-01-31 14:52:20 +0000
@@ -32,7 +32,7 @@
 #include "privatewindow.h"
 
 bool
-CompScreen::closeWin (CompAction         *action,
+CompScreenImpl::closeWin (CompAction         *action,
 		      CompAction::State  state,
 		      CompOption::Vector &options)
 {
@@ -51,7 +51,7 @@
 }
 
 bool
-CompScreen::unmaximizeWin (CompAction         *action,
+CompScreenImpl::unmaximizeWin (CompAction         *action,
 			   CompAction::State  state,
 			   CompOption::Vector &options)
 {
@@ -68,7 +68,7 @@
 }
 
 bool
-CompScreen::minimizeWin (CompAction         *action,
+CompScreenImpl::minimizeWin (CompAction         *action,
 			 CompAction::State  state,
 			 CompOption::Vector &options)
 {
@@ -85,7 +85,7 @@
 }
 
 bool
-CompScreen::maximizeWin (CompAction         *action,
+CompScreenImpl::maximizeWin (CompAction         *action,
 			 CompAction::State  state,
 			 CompOption::Vector &options)
 {
@@ -102,7 +102,7 @@
 }
 
 bool
-CompScreen::maximizeWinHorizontally (CompAction         *action,
+CompScreenImpl::maximizeWinHorizontally (CompAction         *action,
 				     CompAction::State  state,
 				     CompOption::Vector &options)
 {
@@ -119,7 +119,7 @@
 }
 
 bool
-CompScreen::maximizeWinVertically (CompAction         *action,
+CompScreenImpl::maximizeWinVertically (CompAction         *action,
 				   CompAction::State  state,
 				   CompOption::Vector &options)
 {
@@ -136,7 +136,7 @@
 }
 
 bool
-CompScreen::showDesktop (CompAction         *action,
+CompScreenImpl::showDesktop (CompAction         *action,
 			 CompAction::State  state,
 			 CompOption::Vector &options)
 {
@@ -149,7 +149,7 @@
 }
 
 bool
-CompScreen::raiseWin (CompAction         *action,
+CompScreenImpl::raiseWin (CompAction         *action,
 		      CompAction::State  state,
 		      CompOption::Vector &options)
 {
@@ -166,7 +166,7 @@
 }
 
 bool
-CompScreen::lowerWin (CompAction         *action,
+CompScreenImpl::lowerWin (CompAction         *action,
 		      CompAction::State  state,
 		      CompOption::Vector &options)
 {
@@ -183,7 +183,7 @@
 }
 
 bool
-CompScreen::windowMenu (CompAction         *action,
+CompScreenImpl::windowMenu (CompAction         *action,
 			CompAction::State  state,
 			CompOption::Vector &options)
 {
@@ -213,7 +213,7 @@
 }
 
 bool
-CompScreen::toggleWinMaximized (CompAction         *action,
+CompScreenImpl::toggleWinMaximized (CompAction         *action,
 				CompAction::State  state,
 				CompOption::Vector &options)
 {
@@ -235,7 +235,7 @@
 }
 
 bool
-CompScreen::toggleWinMaximizedHorizontally (CompAction         *action,
+CompScreenImpl::toggleWinMaximizedHorizontally (CompAction         *action,
 					    CompAction::State  state,
 					    CompOption::Vector &options)
 {
@@ -252,7 +252,7 @@
 }
 
 bool
-CompScreen::toggleWinMaximizedVertically (CompAction         *action,
+CompScreenImpl::toggleWinMaximizedVertically (CompAction         *action,
 					  CompAction::State  state,
 					  CompOption::Vector &options)
 {
@@ -269,7 +269,7 @@
 }
 
 bool
-CompScreen::shadeWin (CompAction         *action,
+CompScreenImpl::shadeWin (CompAction         *action,
 		      CompAction::State  state,
 		      CompOption::Vector &options)
 {

=== modified file 'src/event.cpp'
--- src/event.cpp	2012-01-26 17:39:42 +0000
+++ src/event.cpp	2012-01-31 14:52:20 +0000
@@ -65,7 +65,7 @@
 
 
 static bool
-autoRaiseTimeout (CompScreen *screen)
+autoRaiseTimeout (CompScreenImpl *screen)
 {
     CompWindow  *w = screen->findWindow (screen->activeWindow ());
 
@@ -975,7 +975,7 @@
 }
 
 void
-AbstractCompScreen::handleCompizEvent (const char         *plugin,
+CompScreen::handleCompizEvent (const char         *plugin,
 			       const char         *event,
 			       CompOption::Vector &options)
 {
@@ -984,21 +984,21 @@
 }
 
 void
-CompScreen::_handleCompizEvent (const char         *plugin,
+CompScreenImpl::_handleCompizEvent (const char         *plugin,
 			       const char         *event,
 			       CompOption::Vector &options)
 {
 }
 
 void
-AbstractCompScreen::handleEvent (XEvent *event)
+CompScreen::handleEvent (XEvent *event)
 {
     WRAPABLE_HND_FUNCTN (handleEvent, event)
     _handleEvent (event);
 }
 
 void
-CompScreen::_handleEvent (XEvent *event)
+CompScreenImpl::_handleEvent (XEvent *event)
 {
     CompWindow *w = NULL;
     XWindowAttributes wa;
@@ -1098,7 +1098,7 @@
 	    }
 	}
 
-	foreach (CompWindow *w, screen->priv->destroyedWindows)
+	foreach (CompWindow *w, priv->destroyedWindows)
 	{
 	    if (w->priv->serverId == event->xcreatewindow.window)
 	    {
@@ -1134,7 +1134,7 @@
 		CoreWindow *cw = new CoreWindow (event->xcreatewindow.window);
 		cw->manage (priv->getTopWindow (), wa);
 
-		priv->createdWindows.remove (cw);
+		removeFromCreatedWindows (cw);
 		delete cw;
             }
 	    else
@@ -1157,7 +1157,7 @@
 
 	if (!w)
 	{
-	    foreach (CompWindow *dw, screen->priv->destroyedWindows)
+	    foreach (CompWindow *dw, priv->destroyedWindows)
 	    {
 		if (dw->priv->serverId == event->xdestroywindow.window)
 		{
@@ -1274,7 +1274,7 @@
 		CoreWindow *cw = new CoreWindow (event->xcreatewindow.window);
 		cw->manage (priv->getTopWindow (), wa);
 
-		priv->createdWindows.remove (cw);
+		removeFromCreatedWindows (cw);
 		delete cw;
 		break;
 	    }
@@ -1286,7 +1286,7 @@
 		 * that it is already in the list of destroyed
 		 * windows, so check that list too */
 
-		foreach (CompWindow *dw, screen->priv->destroyedWindows)
+		foreach (CompWindow *dw, priv->destroyedWindows)
 		{
 		    if (dw->priv->serverId == event->xreparent.window)
 		    {

=== modified file 'src/main.cpp'
--- src/main.cpp	2012-01-23 18:41:06 +0000
+++ src/main.cpp	2012-01-31 14:52:20 +0000
@@ -140,7 +140,8 @@
 bool
 CompManager::init ()
 {
-    screen = new CompScreen ();
+    CompScreenImpl* ss = new CompScreenImpl ();
+    screen = ss;
 
     if (!screen || !screen->priv)
 	return false;
@@ -177,7 +178,7 @@
     screen->priv->dirtyPluginList = true;
     screen->priv->updatePlugins ();
 
-    if (!screen->init (displayName))
+    if (!ss->init (displayName))
 	return false;
 
     if (debugOutput)

=== modified file 'src/match.cpp'
--- src/match.cpp	2012-01-26 17:39:42 +0000
+++ src/match.cpp	2012-01-31 14:52:20 +0000
@@ -112,14 +112,14 @@
 };
 
 CompMatch::Expression *
-AbstractCompScreen::matchInitExp (const CompString& str)
+CompScreen::matchInitExp (const CompString& str)
 {
     WRAPABLE_HND_FUNCTN_RETURN (CompMatch::Expression *, matchInitExp, str);
     return _matchInitExp (str);
 }
 
 CompMatch::Expression *
-CompScreen::_matchInitExp (const CompString& str)
+CompScreenImpl::_matchInitExp (const CompString& str)
 {
     return new CoreExp (str);
 }
@@ -146,14 +146,14 @@
 }
 
 void
-AbstractCompScreen::matchExpHandlerChanged ()
+CompScreen::matchExpHandlerChanged ()
 {
     WRAPABLE_HND_FUNCTN (matchExpHandlerChanged);
     _matchExpHandlerChanged ();
 }
 
 void
-CompScreen::_matchExpHandlerChanged ()
+CompScreenImpl::_matchExpHandlerChanged ()
 {
     foreach (CompPlugin *p, CompPlugin::getPlugins ())
     {
@@ -163,14 +163,14 @@
 }
 
 void
-AbstractCompScreen::matchPropertyChanged (CompWindow *w)
+CompScreen::matchPropertyChanged (CompWindow *w)
 {
     WRAPABLE_HND_FUNCTN (matchPropertyChanged, w);
     _matchPropertyChanged (w);
 }
 
 void
-CompScreen::_matchPropertyChanged (CompWindow *w)
+CompScreenImpl::_matchPropertyChanged (CompWindow *w)
 {
 }
 

=== modified file 'src/option.cpp'
--- src/option.cpp	2012-01-25 13:32:02 +0000
+++ src/option.cpp	2012-01-31 14:52:20 +0000
@@ -417,7 +417,7 @@
 }
 
 static void
-finiScreenOptionValue (CompScreen        *s,
+finiScreenOptionValue (CompScreenImpl        *s,
 		       CompOption::Value &v)
 {
     switch (v.type()) {

=== modified file 'src/plugin.cpp'
--- src/plugin.cpp	2012-01-30 10:17:42 +0000
+++ src/plugin.cpp	2012-01-31 14:52:20 +0000
@@ -320,14 +320,14 @@
 }
 
 bool
-AbstractCompScreen::initPluginForScreen (CompPlugin *p)
+CompScreen::initPluginForScreen (CompPlugin *p)
 {
     WRAPABLE_HND_FUNCTN_RETURN (bool, initPluginForScreen, p)
     return _initPluginForScreen (p);
 }
 
 bool
-CompScreen::_initPluginForScreen (CompPlugin *p)
+CompScreenImpl::_initPluginForScreen (CompPlugin *p)
 {
     bool status               = true;
     CompWindowList::iterator it, fail;
@@ -357,21 +357,21 @@
 }
 
 void
-AbstractCompScreen::finiPluginForScreen (CompPlugin *p)
+CompScreen::finiPluginForScreen (CompPlugin *p)
 {
     WRAPABLE_HND_FUNCTN (finiPluginForScreen, p)
     _finiPluginForScreen (p);
 }
 
 void
-CompScreen::_finiPluginForScreen (CompPlugin *p)
+CompScreenImpl::_finiPluginForScreen (CompPlugin *p)
 {
     foreach (CompWindow *w, priv->windows)
 	p->vTable->finiWindow (w);
 }
 
 bool
-CompPlugin::screenInitPlugins (CompScreen *s)
+CompPlugin::screenInitPlugins (CompScreenImpl *s)
 {
     CompPlugin::List::reverse_iterator it = plugins.rbegin ();
 
@@ -392,7 +392,7 @@
 }
 
 void
-CompPlugin::screenFiniPlugins (CompScreen *s)
+CompPlugin::screenFiniPlugins (CompScreenImpl *s)
 {
     foreach (CompPlugin *p, plugins)
     {

=== modified file 'src/privateiosource.h'
--- src/privateiosource.h	2012-01-20 17:50:31 +0000
+++ src/privateiosource.h	2012-01-31 14:52:20 +0000
@@ -51,7 +51,7 @@
 	bool		  mForceFail;
 	bool		  mExecuting;
 
-    friend class CompScreen;
+    friend class CompScreenImpl;
 };
 
 #endif

=== modified file 'src/privatescreen.h'
--- src/privatescreen.h	2012-01-30 10:17:42 +0000
+++ src/privatescreen.h	2012-01-31 14:52:20 +0000
@@ -203,14 +203,14 @@
 	class Grab {
 	    public:
 
-		friend class CompScreen;
+		friend class CompScreenImpl;
 	    private:
 		Cursor     cursor;
 		const char *name;
 	};
 
     public:
-	PrivateScreen (AbstractCompScreen *screen);
+	PrivateScreen (CompScreen *screen);
 	~PrivateScreen ();
 
 	bool setOption (const CompString &name, CompOption::Value &value);
@@ -447,7 +447,7 @@
 	CompOption::Value plugin;
 	bool	          dirtyPluginList;
 
-	AbstractCompScreen  *screen;
+	CompScreen  *screen;
 
 	std::list <CoreWindow *> createdWindows;
 	CompWindowList serverWindows;

=== modified file 'src/privatescreen/tests/test-privatescreen.cpp'
--- src/privatescreen/tests/test-privatescreen.cpp	2012-01-30 16:42:19 +0000
+++ src/privatescreen/tests/test-privatescreen.cpp	2012-01-31 14:52:20 +0000
@@ -11,7 +11,7 @@
 
 namespace {
 
-class MockCompScreen : public AbstractCompScreen
+class MockCompScreen : public CompScreen
 {
 public:
     // Interface hoisted from CompScreen
@@ -59,6 +59,56 @@
     MOCK_METHOD0(_matchExpHandlerChanged, void ());
     MOCK_METHOD1(_matchPropertyChanged, void (CompWindow *));
     MOCK_METHOD0(_outputChangeNotify, void ());
+
+    MOCK_METHOD0(outputDevs, CompOutput::vector & ());
+    MOCK_METHOD2(setWindowState, void (unsigned int state, Window id));
+    MOCK_METHOD0(XShape, bool ());
+    MOCK_METHOD0(screenInfo, std::vector<XineramaScreenInfo> & ());
+    MOCK_METHOD0(serverWindows, CompWindowList & ());
+    MOCK_METHOD3(setWindowProp, void (Window       id,
+			    Atom         property,
+			    unsigned int value));
+    MOCK_METHOD0(activeWindow, Window ());
+    MOCK_METHOD0(currentDesktop, unsigned int ());
+    MOCK_METHOD0(currentHistory, CompActiveWindowHistory *());
+    MOCK_METHOD0(focusDefaultWindow, void ());
+    MOCK_METHOD0(getCurrentTime, Time ());
+    MOCK_METHOD3(getWindowProp, unsigned int (Window       id,
+				    Atom         property,
+				    unsigned int defaultValue));
+    MOCK_METHOD2(insertServerWindow, void (CompWindow *w, Window aboveId));
+    MOCK_METHOD2(insertWindow, void (CompWindow *w, Window aboveId));
+    MOCK_METHOD0(nDesktop, unsigned int ());
+    MOCK_METHOD1(outputDeviceForGeometry, int (const CompWindow::Geometry& gm));
+    MOCK_METHOD0(screenNum, int ());
+    MOCK_METHOD1(unhookServerWindow, void (CompWindow *w));
+    MOCK_METHOD1(unhookWindow, void (CompWindow *w));
+    MOCK_METHOD2(viewportForGeometry, void (const CompWindow::Geometry &gm,
+				  CompPoint                   &viewport));
+
+    MOCK_METHOD1(removeFromCreatedWindows, void (CoreWindow *cw));
+    MOCK_METHOD1(addToDestroyedWindows, void (CompWindow * cw));
+
+    MOCK_CONST_METHOD0(workArea, CompRect const& ());
+    MOCK_METHOD1(removeAction, void (CompAction *action));
+    MOCK_METHOD0(getOptions, CompOption::Vector & ());
+    MOCK_METHOD2(setOption, bool (const CompString &name, CompOption::Value &value));
+    MOCK_METHOD2(storeValue, void (CompString key, CompPrivate value));
+    MOCK_METHOD1(hasValue, bool (CompString key));
+    MOCK_METHOD1(getValue, CompPrivate (CompString key));
+    MOCK_METHOD1(eraseValue, void (CompString key));
+    MOCK_METHOD3(addWatchFd, CompWatchFdHandle (int             fd,
+				      short int       events,
+				      FdWatchCallBack callBack));
+    MOCK_METHOD1(removeWatchFd, void (CompWatchFdHandle handle));
+    MOCK_METHOD0(eventLoop, void ());
+    MOCK_METHOD3(addFileWatch, CompFileWatchHandle (const char        *path,
+					  int               mask,
+					  FileWatchCallBack callBack));
+    MOCK_METHOD1(removeFileWatch, void (CompFileWatchHandle handle));
+    MOCK_CONST_METHOD0(getFileWatches, const CompFileWatchList& ());
+    MOCK_METHOD0(updateSupportedWmHints, void ());
+
 };
 
 

=== modified file 'src/privatewindow.h'
--- src/privatewindow.h	2011-10-10 14:56:06 +0000
+++ src/privatewindow.h	2012-01-31 14:52:20 +0000
@@ -342,7 +342,7 @@
 
     friend class PrivateWindow;
     friend class PrivateScreen;
-    friend class CompScreen;
+    friend class CompScreenImpl;
 
 private:
 

=== modified file 'src/screen.cpp'
--- src/screen.cpp	2012-01-30 16:42:19 +0000
+++ src/screen.cpp	2012-01-31 14:52:20 +0000
@@ -87,25 +87,40 @@
 } MwmHints;
 
 
-
 #if defined(ARG_ABSTRACT_COMP_SCREEN)
-class AbstractCompScreen;
-AbstractCompScreen   *screen;
-#else
 class CompScreen;
 CompScreen   *screen;
+#else
+class CompScreenImpl;
+CompScreenImpl   *screen;
 #endif
 
 ModifierHandler *modHandler;
 
 PluginClassStorage::Indices screenPluginClassIndices (0);
 
-void CompScreen::sizePluginClasses(unsigned int size)
+void CompScreenImpl::sizePluginClasses(unsigned int size)
 {
     if(size != pluginClasses.size ())
  	pluginClasses.resize (size);
 }
 
+void CompScreenImpl::setWindowState (unsigned int state, Window id)
+{
+    priv->setWindowState (state, id);
+}
+
+void
+CompScreenImpl::removeFromCreatedWindows(CoreWindow *cw)
+{
+    priv->createdWindows.remove (cw);
+}
+
+void CompScreenImpl::addToDestroyedWindows(CompWindow * cw)
+{
+    priv->destroyedWindows.push_back (cw);
+}
+
 
 unsigned int
 CompScreen::allocPluginClassIndex ()
@@ -146,7 +161,7 @@
 }
 
 void
-CompScreen::eventLoop ()
+CompScreenImpl::eventLoop ()
 {
     priv->source = CompEventSource::create ();
     priv->timeout = CompTimeoutSource::create (priv->ctx);
@@ -159,7 +174,7 @@
 }
 
 CompFileWatchHandle
-CompScreen::addFileWatch (const char        *path,
+CompScreenImpl::addFileWatch (const char        *path,
 			  int               mask,
 			  FileWatchCallBack callBack)
 {
@@ -183,7 +198,7 @@
 }
 
 void
-CompScreen::removeFileWatch (CompFileWatchHandle handle)
+CompScreenImpl::removeFileWatch (CompFileWatchHandle handle)
 {
     std::list<CompFileWatch *>::iterator it;
     CompFileWatch                        *w;
@@ -204,7 +219,7 @@
 }
 
 const CompFileWatchList &
-CompScreen::getFileWatches () const
+CompScreenImpl::getFileWatches () const
 {
     return priv->fileWatch;
 }
@@ -235,7 +250,7 @@
 }
 
 CompWatchFdHandle
-CompScreen::addWatchFd (int             fd,
+CompScreenImpl::addWatchFd (int             fd,
 			short int       events,
 			FdWatchCallBack callBack)
 {
@@ -271,7 +286,7 @@
 }
 
 void
-CompScreen::removeWatchFd (CompWatchFdHandle handle)
+CompScreenImpl::removeWatchFd (CompWatchFdHandle handle)
 {
     std::list<CompWatchFd * >::iterator it;
     CompWatchFd *			w;
@@ -299,19 +314,19 @@
 }
 
 void
-CompScreen::storeValue (CompString key, CompPrivate value)
+CompScreenImpl::storeValue (CompString key, CompPrivate value)
 {
     ValueHolder::Default ()->storeValue (key, value);
 }
 
 bool
-CompScreen::hasValue (CompString key)
+CompScreenImpl::hasValue (CompString key)
 {
     return ValueHolder::Default ()->hasValue (key);
 }
 
 CompPrivate
-CompScreen::getValue (CompString key)
+CompScreenImpl::getValue (CompString key)
 {
     return ValueHolder::Default ()->getValue (key);
 }
@@ -350,37 +365,37 @@
 }    
 
 void
-CompScreen::eraseValue (CompString key)
+CompScreenImpl::eraseValue (CompString key)
 {
     ValueHolder::Default ()->eraseValue (key);
 }
 
 void
-AbstractCompScreen::fileWatchAdded (CompFileWatch *watch)
+CompScreen::fileWatchAdded (CompFileWatch *watch)
 {
     WRAPABLE_HND_FUNCTN (fileWatchAdded, watch);
     _fileWatchAdded (watch);
 }
 
 void
-CompScreen::_fileWatchAdded (CompFileWatch *watch)
+CompScreenImpl::_fileWatchAdded (CompFileWatch *watch)
 {
 }
 
 void
-AbstractCompScreen::fileWatchRemoved (CompFileWatch *watch)
+CompScreen::fileWatchRemoved (CompFileWatch *watch)
 {
     WRAPABLE_HND_FUNCTN (fileWatchRemoved, watch);
     _fileWatchRemoved (watch);
 }
 
 void
-CompScreen::_fileWatchRemoved (CompFileWatch *watch)
+CompScreenImpl::_fileWatchRemoved (CompFileWatch *watch)
 {
 }
 
 bool
-AbstractCompScreen::setOptionForPlugin (const char        *plugin,
+CompScreen::setOptionForPlugin (const char        *plugin,
 				const char        *name,
 				CompOption::Value &value)
 {
@@ -391,7 +406,7 @@
 }
 
 bool
-CompScreen::_setOptionForPlugin (const char        *plugin,
+CompScreenImpl::_setOptionForPlugin (const char        *plugin,
 				const char        *name,
 				CompOption::Value &value)
 {
@@ -403,7 +418,7 @@
 }
 
 void
-AbstractCompScreen::sessionEvent (CompSession::Event event,
+CompScreen::sessionEvent (CompSession::Event event,
 			  CompOption::Vector &arguments)
 {
     WRAPABLE_HND_FUNCTN (sessionEvent, event, arguments);
@@ -411,7 +426,7 @@
 }
 
 void
-CompScreen::_sessionEvent (CompSession::Event event,
+CompScreenImpl::_sessionEvent (CompSession::Event event,
 			  CompOption::Vector &arguments)
 {
 }
@@ -488,7 +503,7 @@
 }
 
 int
-CompScreen::checkForError (Display *dpy)
+CompScreenImpl::checkForError (Display *dpy)
 {
     int e;
 
@@ -501,61 +516,61 @@
 }
 
 Display *
-CompScreen::dpy ()
+CompScreenImpl::dpy ()
 {
     return priv->dpy;
 }
 
 bool
-CompScreen::XRandr ()
+CompScreenImpl::XRandr ()
 {
     return priv->randrExtension;
 }
 
 int
-CompScreen::randrEvent ()
+CompScreenImpl::randrEvent ()
 {
     return priv->randrEvent;
 }
 
 bool
-CompScreen::XShape ()
+CompScreenImpl::XShape ()
 {
     return priv->shapeExtension;
 }
 
 int
-CompScreen::shapeEvent ()
+CompScreenImpl::shapeEvent ()
 {
     return priv->shapeEvent;
 }
 
 int
-CompScreen::syncEvent ()
+CompScreenImpl::syncEvent ()
 {
     return priv->syncEvent;
 }
 
 SnDisplay *
-CompScreen::snDisplay ()
+CompScreenImpl::snDisplay ()
 {
     return priv->snDisplay;
 }
 
 Window
-CompScreen::activeWindow ()
+CompScreenImpl::activeWindow ()
 {
     return priv->activeWindow;
 }
 
 Window
-CompScreen::autoRaiseWindow ()
+CompScreenImpl::autoRaiseWindow ()
 {
     return priv->autoRaiseWindow;
 }
 
 const char *
-CompScreen::displayString ()
+CompScreenImpl::displayString ()
 {
     return priv->displayString;
 }
@@ -618,13 +633,13 @@
 }
 
 CompOption::Vector &
-CompScreen::getOptions ()
+CompScreenImpl::getOptions ()
 {
     return priv->getOptions ();
 }
 
 bool
-CompScreen::setOption (const CompString  &name,
+CompScreenImpl::setOption (const CompString  &name,
 		       CompOption::Value &value)
 {
     return priv->setOption (name, value);
@@ -1164,7 +1179,7 @@
 #define HOMECOMPIZDIR ".compiz-1"
 
 bool
-CompScreen::readImageFromFile (CompString &name,
+CompScreenImpl::readImageFromFile (CompString &name,
 			       CompString &pname,
 			       CompSize   &size,
 			       void       *&data)
@@ -1209,7 +1224,7 @@
 }
 
 bool
-CompScreen::writeImageToFile (CompString &path,
+CompScreenImpl::writeImageToFile (CompString &path,
 			      const char *format,
 			      CompSize   &size,
 			      void       *data)
@@ -1243,7 +1258,7 @@
 }
 
 bool
-AbstractCompScreen::fileToImage (CompString &name,
+CompScreen::fileToImage (CompString &name,
 			 CompSize   &size,
 			 int        &stride,
 			 void       *&data)
@@ -1253,7 +1268,7 @@
 }
 
 bool
-CompScreen::_fileToImage (CompString &name,
+CompScreenImpl::_fileToImage (CompString &name,
 			 CompSize   &size,
 			 int        &stride,
 			 void       *&data)
@@ -1262,7 +1277,7 @@
 }
 
 bool
-AbstractCompScreen::imageToFile (CompString &path,
+CompScreen::imageToFile (CompString &path,
 			 CompString &format,
 			 CompSize   &size,
 			 int        stride,
@@ -1274,7 +1289,7 @@
 }
 
 bool
-CompScreen::_imageToFile (CompString &path,
+CompScreenImpl::_imageToFile (CompString &path,
 			 CompString &format,
 			 CompSize   &size,
 			 int        stride,
@@ -1284,7 +1299,7 @@
 }
 
 void
-AbstractCompScreen::logMessage (const char   *componentName,
+CompScreen::logMessage (const char   *componentName,
 			CompLogLevel level,
 			const char   *message)
 {
@@ -1293,7 +1308,7 @@
 }
 
 void
-CompScreen::_logMessage (const char   *componentName,
+CompScreenImpl::_logMessage (const char   *componentName,
 			CompLogLevel level,
 			const char   *message)
 {
@@ -1588,7 +1603,7 @@
 }
 
 unsigned int
-CompScreen::getWindowProp (Window       id,
+CompScreenImpl::getWindowProp (Window       id,
 			   Atom         property,
 			   unsigned int defaultValue)
 {
@@ -1618,7 +1633,7 @@
 }
 
 void
-CompScreen::setWindowProp (Window       id,
+CompScreenImpl::setWindowProp (Window       id,
 			   Atom         property,
 			   unsigned int value)
 {
@@ -1662,7 +1677,7 @@
 }
 
 unsigned short
-CompScreen::getWindowProp32 (Window         id,
+CompScreenImpl::getWindowProp32 (Window         id,
 			     Atom           property,
 			     unsigned short defaultValue)
 {
@@ -1675,7 +1690,7 @@
 }
 
 void
-CompScreen::setWindowProp32 (Window         id,
+CompScreenImpl::setWindowProp32 (Window         id,
 			     Atom           property,
 			     unsigned short value)
 {
@@ -2143,10 +2158,10 @@
 }
 
 void
-CompScreen::compScreenSnEvent (SnMonitorEvent *event,
+CompScreenImpl::compScreenSnEvent (SnMonitorEvent *event,
 			       void           *userData)
 {
-    CompScreen	      *screen = (CompScreen *) userData;
+    CompScreenImpl	      *screen = (CompScreenImpl *) userData;
     SnStartupSequence *sequence;
 
     sequence = sn_monitor_event_get_startup_sequence (event);
@@ -2269,7 +2284,7 @@
 }
 
 void
-CompScreen::updateSupportedWmHints ()
+CompScreenImpl::updateSupportedWmHints ()
 {
     std::vector<Atom> atoms;
 
@@ -2281,14 +2296,14 @@
 }
 
 void
-AbstractCompScreen::addSupportedAtoms (std::vector<Atom> &atoms)
+CompScreen::addSupportedAtoms (std::vector<Atom> &atoms)
 {
     WRAPABLE_HND_FUNCTN (addSupportedAtoms, atoms);
     _addSupportedAtoms (atoms);
 }
 
 void
-CompScreen::_addSupportedAtoms (std::vector<Atom> &atoms)
+CompScreenImpl::_addSupportedAtoms (std::vector<Atom> &atoms)
 {
     atoms.push_back (Atoms::supported);
     atoms.push_back (Atoms::supportingWmCheck);
@@ -2486,24 +2501,24 @@
 }
 
 void
-AbstractCompScreen::enterShowDesktopMode ()
+CompScreen::enterShowDesktopMode ()
 {
     WRAPABLE_HND_FUNCTN (enterShowDesktopMode)
     _enterShowDesktopMode ();
 }
 
-unsigned int CompScreen::showingDesktopMask() const
+unsigned int CompScreenImpl::showingDesktopMask() const
 {
     return priv->showingDesktopMask;
 }
 
-bool CompScreen::grabsEmpty() const
+bool CompScreenImpl::grabsEmpty() const
 {
     return priv->grabs.empty();
 }
 
 void
-CompScreen::_enterShowDesktopMode ()
+CompScreenImpl::_enterShowDesktopMode ()
 {
     unsigned long data = 1;
     int		  count = 0;
@@ -2543,14 +2558,14 @@
 }
 
 void
-AbstractCompScreen::leaveShowDesktopMode (CompWindow *window)
+CompScreen::leaveShowDesktopMode (CompWindow *window)
 {
     WRAPABLE_HND_FUNCTN (leaveShowDesktopMode, window)
     _leaveShowDesktopMode (window);
 }
 
 void
-CompScreen::_leaveShowDesktopMode (CompWindow *window)
+CompScreenImpl::_leaveShowDesktopMode (CompWindow *window)
 {
     unsigned long data = 0;
 
@@ -2596,14 +2611,14 @@
 }
 
 void
-CompScreen::forEachWindow (CompWindow::ForEach proc)
+CompScreenImpl::forEachWindow (CompWindow::ForEach proc)
 {
     foreach (CompWindow *w, priv->windows)
 	proc (w);
 }
 
 void
-CompScreen::focusDefaultWindow ()
+CompScreenImpl::focusDefaultWindow ()
 {
     CompWindow  *w;
     CompWindow  *focus = NULL;
@@ -2687,7 +2702,7 @@
 }
 
 CompWindow *
-CompScreen::findWindow (Window id)
+CompScreenImpl::findWindow (Window id)
 {
     if (lastFoundWindow && lastFoundWindow->id () == id)
     {
@@ -2705,7 +2720,7 @@
 }
 
 CompWindow *
-CompScreen::findTopLevelWindow (Window id, bool override_redirect)
+CompScreenImpl::findTopLevelWindow (Window id, bool override_redirect)
 {
     CompWindow *w;
 
@@ -2732,7 +2747,7 @@
 }
 
 void
-CompScreen::insertWindow (CompWindow *w, Window	aboveId)
+CompScreenImpl::insertWindow (CompWindow *w, Window	aboveId)
 {
     StackDebugger *dbg = StackDebugger::Default ();
 
@@ -2795,7 +2810,7 @@
 }
 
 void
-CompScreen::insertServerWindow (CompWindow *w, Window	aboveId)
+CompScreenImpl::insertServerWindow (CompWindow *w, Window	aboveId)
 {
     StackDebugger *dbg = StackDebugger::Default ();
 
@@ -2859,7 +2874,7 @@
 }
 
 void
-CompScreen::unhookWindow (CompWindow *w)
+CompScreenImpl::unhookWindow (CompWindow *w)
 {
     StackDebugger *dbg = StackDebugger::Default ();
 
@@ -2892,7 +2907,7 @@
 }
 
 void
-CompScreen::unhookServerWindow (CompWindow *w)
+CompScreenImpl::unhookServerWindow (CompWindow *w)
 {
     StackDebugger *dbg = StackDebugger::Default ();
 
@@ -2921,13 +2936,13 @@
 }
 
 Cursor
-CompScreen::normalCursor ()
+CompScreenImpl::normalCursor ()
 {
     return priv->normalCursor;
 }
 
 Cursor
-CompScreen::invisibleCursor ()
+CompScreenImpl::invisibleCursor ()
 {
     return priv->invisibleCursor;
 }
@@ -2935,8 +2950,8 @@
 #define POINTER_GRAB_MASK (ButtonReleaseMask | \
 			   ButtonPressMask   | \
 			   PointerMotionMask)
-CompScreen::GrabHandle
-CompScreen::pushGrab (Cursor cursor, const char *name)
+CompScreenImpl::GrabHandle
+CompScreenImpl::pushGrab (Cursor cursor, const char *name)
 {
     if (priv->grabs.empty ())
     {
@@ -2979,7 +2994,7 @@
 }
 
 void
-CompScreen::updateGrab (CompScreen::GrabHandle handle, Cursor cursor)
+CompScreenImpl::updateGrab (CompScreenImpl::GrabHandle handle, Cursor cursor)
 {
     if (!handle)
 	return;
@@ -2991,7 +3006,7 @@
 }
 
 void
-CompScreen::removeGrab (CompScreen::GrabHandle handle,
+CompScreenImpl::removeGrab (CompScreenImpl::GrabHandle handle,
 			CompPoint *restorePointer)
 {
     if (!handle)
@@ -3029,7 +3044,7 @@
    plugins listed, returns false otherwise. */
 
 bool
-CompScreen::otherGrabExist (const char *first, ...)
+CompScreenImpl::otherGrabExist (const char *first, ...)
 {
     va_list    ap;
     const char *name;
@@ -3059,7 +3074,7 @@
 }
 
 bool
-CompScreen::grabExist (const char *grab)
+CompScreenImpl::grabExist (const char *grab)
 {
     foreach (PrivateScreen::Grab* g, priv->grabs)
     {
@@ -3070,7 +3085,7 @@
 }
 
 bool
-CompScreen::grabbed ()
+CompScreenImpl::grabbed ()
 {
     return priv->grabbed;
 }
@@ -3107,7 +3122,7 @@
     int             mod, k;
     unsigned int    ignore;
 
-    CompScreen::checkForError (dpy);
+    CompScreenImpl::checkForError (dpy);
 
     for (ignore = 0; ignore <= modHandler->ignoredModMask (); ignore++)
     {
@@ -3140,7 +3155,7 @@
 	    }
 	}
 
-	if (CompScreen::checkForError (dpy))
+	if (CompScreenImpl::checkForError (dpy))
 	    return false;
     }
 
@@ -3291,7 +3306,7 @@
 }
 
 bool
-CompScreen::addAction (CompAction *action)
+CompScreenImpl::addAction (CompAction *action)
 {
     if (!screenInitalized || !priv->initialized)
 	return false;
@@ -3331,7 +3346,7 @@
 }
 
 void
-CompScreen::removeAction (CompAction *action)
+CompScreenImpl::removeAction (CompAction *action)
 {
     if (!priv->initialized)
 	return;
@@ -3417,7 +3432,7 @@
 }
 
 void
-CompScreen::updateWorkarea ()
+CompScreenImpl::updateWorkarea ()
 {
     CompRect workArea;
     CompRegion allWorkArea = CompRegion ();
@@ -3584,13 +3599,13 @@
 }
 
 const CompWindowVector &
-CompScreen::clientList (bool stackingOrder)
+CompScreenImpl::clientList (bool stackingOrder)
 {
    return stackingOrder ? priv->clientListStacking : priv->clientList;
 }
 
 void
-CompScreen::toolkitAction (Atom   toolkitAction,
+CompScreenImpl::toolkitAction (Atom   toolkitAction,
 			   Time   eventTime,
 			   Window window,
 			   long   data0,
@@ -3617,7 +3632,7 @@
 }
 
 void
-CompScreen::runCommand (CompString command)
+CompScreenImpl::runCommand (CompString command)
 {
     if (command.size () == 0)
 	return;
@@ -3655,7 +3670,7 @@
 }
 
 void
-CompScreen::moveViewport (int tx, int ty, bool sync)
+CompScreenImpl::moveViewport (int tx, int ty, bool sync)
 {
     CompPoint pnt;
 
@@ -3797,7 +3812,7 @@
 }
 
 void
-CompScreen::sendWindowActivationRequest (Window id)
+CompScreenImpl::sendWindowActivationRequest (Window id)
 {
     XEvent xev;
 
@@ -3845,13 +3860,13 @@
 }
 
 int
-CompScreen::outputDeviceForPoint (const CompPoint &point)
+CompScreenImpl::outputDeviceForPoint (const CompPoint &point)
 {
     return outputDeviceForPoint (point.x (), point.y ());
 }
 
 int
-CompScreen::outputDeviceForPoint (int x, int y)
+CompScreenImpl::outputDeviceForPoint (int x, int y)
 {
     CompWindow::Geometry geom (x, y, 1, 1, 0);
 
@@ -3859,7 +3874,7 @@
 }
 
 CompRect
-CompScreen::getCurrentOutputExtents ()
+CompScreenImpl::getCurrentOutputExtents ()
 {
     return priv->outputDevs[priv->currentOutputDev];
 }
@@ -3922,20 +3937,20 @@
 }
 
 const CompRect&
-CompScreen::getWorkareaForOutput (unsigned int outputNum) const
+CompScreenImpl::getWorkareaForOutput (unsigned int outputNum) const
 {
     return priv->outputDevs[outputNum].workArea ();
 }
 
 void
-AbstractCompScreen::outputChangeNotify ()
+CompScreen::outputChangeNotify ()
 {
     WRAPABLE_HND_FUNCTN (outputChangeNotify);
     _outputChangeNotify ();
 }
 
 void
-CompScreen::_outputChangeNotify ()
+CompScreenImpl::_outputChangeNotify ()
 {
 }
 
@@ -3945,7 +3960,7 @@
    is currently computed as the viewport where the center of the window is
    located. */
 void
-CompScreen::viewportForGeometry (const CompWindow::Geometry& gm,
+CompScreenImpl::viewportForGeometry (const CompWindow::Geometry& gm,
 				 CompPoint&                  viewport)
 {
     CompRect rect (gm);
@@ -3964,7 +3979,7 @@
 }
 
 int
-CompScreen::outputDeviceForGeometry (const CompWindow::Geometry& gm)
+CompScreenImpl::outputDeviceForGeometry (const CompWindow::Geometry& gm)
 {
     int          overlapAreas[priv->outputDevs.size ()];
     int          highest, seen, highestScore;
@@ -4078,13 +4093,13 @@
 }
 
 CompIcon *
-CompScreen::defaultIcon () const
+CompScreenImpl::defaultIcon () const
 {
     return priv->defaultIcon;
 }
 
 bool
-CompScreen::updateDefaultIcon ()
+CompScreenImpl::updateDefaultIcon ()
 {
     CompString file = priv->optionGetDefaultIcon ();
     CompString pname = "core/";
@@ -4177,19 +4192,19 @@
 
 
 Window
-CompScreen::root ()
+CompScreenImpl::root ()
 {
     return priv->root;
 }
 
 int
-CompScreen::xkbEvent ()
+CompScreenImpl::xkbEvent ()
 {
     return priv->xkbEvent;
 }
 
 void
-CompScreen::warpPointer (int dx,
+CompScreenImpl::warpPointer (int dx,
 			 int dy)
 {
     XEvent      event;
@@ -4261,26 +4276,26 @@
 }
 
 CompWindowList &
-CompScreen::windows ()
+CompScreenImpl::windows ()
 {
     return priv->windows;
 }
 
 CompWindowList &
-CompScreen::serverWindows ()
+CompScreenImpl::serverWindows ()
 {
     return priv->serverWindows;
 }
 
 CompWindowList &
-CompScreen::destroyedWindows ()
+CompScreenImpl::destroyedWindows ()
 {
     return priv->destroyedWindows;
 }
 
 
 Time
-CompScreen::getCurrentTime ()
+CompScreenImpl::getCurrentTime ()
 {
     XEvent event;
 
@@ -4295,79 +4310,79 @@
 }
 
 Window
-CompScreen::selectionWindow ()
+CompScreenImpl::selectionWindow ()
 {
     return priv->wmSnSelectionWindow;
 }
 
 int
-CompScreen::screenNum ()
+CompScreenImpl::screenNum ()
 {
     return priv->screenNum;
 }
 
 const CompPoint &
-CompScreen::vp () const
+CompScreenImpl::vp () const
 {
     return priv->vp;
 }
 
 const CompSize &
-CompScreen::vpSize () const
+CompScreenImpl::vpSize () const
 {
     return priv->vpSize;
 }
 
 int
-CompScreen::desktopWindowCount ()
+CompScreenImpl::desktopWindowCount ()
 {
     return priv->desktopWindowCount;
 }
 
 unsigned int
-CompScreen::activeNum () const
+CompScreenImpl::activeNum () const
 {
     return priv->activeNum;
 }
 
 CompOutput::vector &
-CompScreen::outputDevs ()
+CompScreenImpl::outputDevs ()
 {
     return priv->outputDevs;
 }
 
 CompOutput &
-CompScreen::currentOutputDev () const
+CompScreenImpl::currentOutputDev () const
 {
     return priv->outputDevs [priv->currentOutputDev];
 }
 
 const CompRect &
-CompScreen::workArea () const
+CompScreenImpl::workArea () const
 {
     return priv->workArea;
 }
 
 unsigned int
-CompScreen::currentDesktop ()
+CompScreenImpl::currentDesktop ()
 {
     return priv->currentDesktop;
 }
 
 unsigned int
-CompScreen::nDesktop ()
+CompScreenImpl::nDesktop ()
 {
     return priv->nDesktop;
 }
 
 CompActiveWindowHistory *
-CompScreen::currentHistory ()
+CompScreenImpl::currentHistory ()
 {
     return &priv->history[priv->currentHistory];
 }
 
 bool
-CompScreen::shouldSerializePlugins ()
+CompScreenImpl::shouldSerializePlugins ()
 {
     return priv->optionGetDoSerialize ();
 }
@@ -4391,32 +4406,32 @@
 }
 
 const CompRegion &
-CompScreen::region () const
+CompScreenImpl::region () const
 {
     return priv->region;
 }
 
 bool
-CompScreen::hasOverlappingOutputs ()
+CompScreenImpl::hasOverlappingOutputs ()
 {
     return priv->hasOverlappingOutputs;
 }
 
 CompOutput &
-CompScreen::fullscreenOutput ()
+CompScreenImpl::fullscreenOutput ()
 {
     return priv->fullscreenOutput;
 }
 
 
 XWindowAttributes
-CompScreen::attrib ()
+CompScreenImpl::attrib ()
 {
     return priv->attrib;
 }
 
 std::vector<XineramaScreenInfo> &
-CompScreen::screenInfo ()
+CompScreenImpl::screenInfo ()
 {
     return priv->screenInfo;
 }
@@ -4429,15 +4444,16 @@
 
 CompScreen::CompScreen ():
     PluginClassStorage (screenPluginClassIndices),
-    priv (NULL)
+    priv (new PrivateScreen (this))
+{
+}
+
+CompScreenImpl::CompScreenImpl ()
 {
     CompPrivate p;
     CompOption::Value::Vector vList;
     CompPlugin  *corePlugin;
 
-    priv = new PrivateScreen (this);
-    assert (priv);
-
     screenInitalized = true;
 
     corePlugin = CompPlugin::load ("core");
@@ -4464,7 +4480,7 @@
 }
 
 bool
-CompScreen::init (const char *name)
+CompScreenImpl::init (const char *name)
 {
     Window               focus;
     int                  revertTo, i;
@@ -4642,7 +4658,7 @@
 
     modHandler->updateModifierMappings ();
 
-    CompScreen::checkForError (dpy);
+    CompScreenImpl::checkForError (dpy);
 
     XGrabServer (dpy);
 
@@ -4683,7 +4699,7 @@
 		      SubstructureNotifyMask);
     }
 
-    if (CompScreen::checkForError (dpy))
+    if (CompScreenImpl::checkForError (dpy))
     {
 	compLogMessage ("core", CompLogLevelError,
 			"Another window manager is "
@@ -4846,7 +4862,7 @@
 	CoreWindow *cw = new CoreWindow (children[i]);
 	cw->manage (i ? children[i - 1] : 0, attrib);
 	delete cw;
-	priv->createdWindows.remove (cw);
+	removeFromCreatedWindows (cw);
     }
 
     /* enforce restack on all windows
@@ -4923,7 +4939,7 @@
     return true;
 }
 
-CompScreen::~CompScreen ()
+CompScreenImpl::~CompScreenImpl ()
 {
     CompPlugin  *p;
 
@@ -4966,7 +4982,7 @@
     screen = NULL;
 }
 
-PrivateScreen::PrivateScreen (AbstractCompScreen *screen) :
+PrivateScreen::PrivateScreen (CompScreen *screen) :
     priv (this),
     fileWatch (0),
     lastFileWatchHandle (1),
@@ -5028,38 +5044,38 @@
 
     memset (&history, 0, sizeof (Window) * ACTIVE_WINDOW_HISTORY_NUM);
 
-    optionSetCloseWindowKeyInitiate (CompScreen::closeWin);
-    optionSetCloseWindowButtonInitiate (CompScreen::closeWin);
-    optionSetRaiseWindowKeyInitiate (CompScreen::raiseWin);
-    optionSetRaiseWindowButtonInitiate (CompScreen::raiseWin);
-    optionSetLowerWindowKeyInitiate (CompScreen::lowerWin);
-    optionSetLowerWindowButtonInitiate (CompScreen::lowerWin);
-
-    optionSetUnmaximizeWindowKeyInitiate (CompScreen::unmaximizeWin);
-
-    optionSetMinimizeWindowKeyInitiate (CompScreen::minimizeWin);
-    optionSetMinimizeWindowButtonInitiate (CompScreen::minimizeWin);
-    optionSetMaximizeWindowKeyInitiate (CompScreen::maximizeWin);
+    optionSetCloseWindowKeyInitiate (CompScreenImpl::closeWin);
+    optionSetCloseWindowButtonInitiate (CompScreenImpl::closeWin);
+    optionSetRaiseWindowKeyInitiate (CompScreenImpl::raiseWin);
+    optionSetRaiseWindowButtonInitiate (CompScreenImpl::raiseWin);
+    optionSetLowerWindowKeyInitiate (CompScreenImpl::lowerWin);
+    optionSetLowerWindowButtonInitiate (CompScreenImpl::lowerWin);
+
+    optionSetUnmaximizeWindowKeyInitiate (CompScreenImpl::unmaximizeWin);
+
+    optionSetMinimizeWindowKeyInitiate (CompScreenImpl::minimizeWin);
+    optionSetMinimizeWindowButtonInitiate (CompScreenImpl::minimizeWin);
+    optionSetMaximizeWindowKeyInitiate (CompScreenImpl::maximizeWin);
     optionSetMaximizeWindowHorizontallyKeyInitiate (
-	CompScreen::maximizeWinHorizontally);
+	CompScreenImpl::maximizeWinHorizontally);
     optionSetMaximizeWindowVerticallyKeyInitiate (
-	CompScreen::maximizeWinVertically);
-
-    optionSetWindowMenuKeyInitiate (CompScreen::windowMenu);
-    optionSetWindowMenuButtonInitiate (CompScreen::windowMenu);
-
-    optionSetShowDesktopKeyInitiate (CompScreen::showDesktop);
-    optionSetShowDesktopEdgeInitiate (CompScreen::showDesktop);
-
-    optionSetToggleWindowMaximizedKeyInitiate (CompScreen::toggleWinMaximized);
-    optionSetToggleWindowMaximizedButtonInitiate (CompScreen::toggleWinMaximized);
+	CompScreenImpl::maximizeWinVertically);
+
+    optionSetWindowMenuKeyInitiate (CompScreenImpl::windowMenu);
+    optionSetWindowMenuButtonInitiate (CompScreenImpl::windowMenu);
+
+    optionSetShowDesktopKeyInitiate (CompScreenImpl::showDesktop);
+    optionSetShowDesktopEdgeInitiate (CompScreenImpl::showDesktop);
+
+    optionSetToggleWindowMaximizedKeyInitiate (CompScreenImpl::toggleWinMaximized);
+    optionSetToggleWindowMaximizedButtonInitiate (CompScreenImpl::toggleWinMaximized);
 
     optionSetToggleWindowMaximizedHorizontallyKeyInitiate (
-	CompScreen::toggleWinMaximizedHorizontally);
+	CompScreenImpl::toggleWinMaximizedHorizontally);
     optionSetToggleWindowMaximizedVerticallyKeyInitiate (
-	CompScreen::toggleWinMaximizedVertically);
+	CompScreenImpl::toggleWinMaximizedVertically);
 
-    optionSetToggleWindowShadedKeyInitiate (CompScreen::shadeWin);
+    optionSetToggleWindowShadedKeyInitiate (CompScreenImpl::shadeWin);
 
     TimeoutHandler::SetDefault (dTimeoutHandler);
 }

=== modified file 'src/window.cpp'
--- src/window.cpp	2012-01-30 16:42:19 +0000
+++ src/window.cpp	2012-01-31 14:52:20 +0000
@@ -504,7 +504,7 @@
     recalcActions ();
 
     if (priv->managed)
-	screen->priv->setWindowState (priv->state, priv->id);
+	screen->setWindowState (priv->state, priv->id);
 
     stateChangeNotify (oldState);
     screen->matchPropertyChanged (this);
@@ -1464,7 +1464,7 @@
 	     * in the stack */
 	    CoreWindow *cw = new CoreWindow (priv->serverFrame);
 	    cw->manage (priv->id, attrib);
-	    screen->priv->createdWindows.remove (cw);
+	    screen->removeFromCreatedWindows (cw);
 	    delete cw;
 	}
 
@@ -1494,7 +1494,7 @@
 	serverNext = oldServerNext;
 	serverPrev = oldServerPrev;
 
-	screen->priv->destroyedWindows.push_back (this);
+	screen->addToDestroyedWindows (this);
 
 	/* We must set the xid of this window
 	 * to zero as it no longer references
@@ -1925,7 +1925,7 @@
 
 	values.events = true;
 
-	CompScreen::checkForError (screen->dpy ());
+	CompScreenImpl::checkForError (screen->dpy ());
 
 	/* Note that by default, the alarm increments the trigger value
 	 * when it fires until the condition (counter.value < trigger.value)
@@ -1940,7 +1940,7 @@
 				      XSyncCAEvents,
 				      &values);
 
-	if (CompScreen::checkForError (screen->dpy ()))
+	if (CompScreenImpl::checkForError (screen->dpy ()))
 	    return true;
 
 	XSyncDestroyAlarm (screen->dpy (), syncAlarm);
@@ -7220,7 +7220,7 @@
 	 * in the stack */
 	CoreWindow *cw = new CoreWindow (serverFrame);
 	CompWindow *fw = cw->manage (id, attrib);
-	screen->priv->createdWindows.remove (cw);
+	screen->removeFromCreatedWindows (cw);
 	delete cw;
 
 	/* Put this window in the list of "detached frame windows"

